from django.db import models
from django.db.models.options import override
from django.utils.translation import gettext as _

import logging
logger = logging.getLogger(__name__)

class Keyword(models.Model):
    """
    this is the model that should contain searchable keywords
    """
    text_de = models.CharField(max_length=40)
    text_en = models.CharField(max_length=40)

    def __str__(self):
        return f"{{<{self.__class__.__name__}>\"{self.text_en}\"}}"

    class Meta:
        verbose_name = _("Keyword")
        verbose_name_plural = _("keywords")

class Searchable(models.Model):
    """
    Abstract class for any model that should be searchable.
    This model will be searched for by the searchbox that is in every upper part of the website.

    This class is not a real abstract class, I need to query it in the main search, so thats impossible
    """
    title_de = models.CharField(max_length=50, default="Titel DE")
    title_en = models.CharField(max_length=50, default="title en")
    subtitle_de = models.CharField(max_length=50, blank=True)
    subtitle_en = models.CharField(max_length=50, blank=True)
    desc_de = models.CharField(max_length=250, unique=True, default="Beschreibung DE")
    desc_en = models.CharField(max_length=250, unique=True, default="Description EN")
    # may be empty/blank for some entries
    date = models.DateField(blank=True, null=True)
    keywords = models.ManyToManyField(Keyword)
    suburl = models.CharField(max_length=200, blank=True, null=True)
    public = models.BooleanField(default=True)

    @classmethod
    def regenerate_all_entries(cls):
        """
        regenerate all searchable items
        """
        logger.info(f"regenerating all {Searchable.__name__} entries")
        for obj in cls.objects.all():
            obj.regenerate()

    def __str__(self):
        return f"{{<{self.__class__.__name__}>\"{self.title_en}\"}}"

    def regenerate(self):
        """
        regenerate a object
        """
        raise NotImplementedError(f"{self.__class__.__name__} does not implement regenerate")

    class Meta:
        verbose_name = _("Searchable")
        verbose_name_plural = _("Searchables")

class StaticSite(Searchable):
    """
    This model represents any static site, such as start:index, 
    that should show up in search.

    Every searchable view should inherit from start.views.SearchableView.
    # TODO automate scanning for SearchableView classes
    """
    
    def regenerate(self):
        """
        regenerate a object
        """
        logger.info(f"regenerating {self.__class__.__name__} object: {self}")
        logger.warning(f"{self.__class__.__name__} cannot regenerate.")
        #self.save()
        pass

    class Meta:
        verbose_name = _("static site")
        verbose_name_plural = _("static sites")
